#ifndef __JABYENGINE_INTERNAL_GPU_HPP__
#define __JABYENGINE_INTERNAL_GPU_HPP__
#include <PSX/GPU/GPU_Types.hpp>
#include <PSX/System/IOPorts/DMA_IO.hpp>
#include <PSX/System/IOPorts/GPU_IO.hpp>

namespace GPU {
    static void quick_fill_fast(const Color24& color, const PositionU16& pos, const SizeU16& size) {
        Port::GP0.write(Port::Command::GP0::QuickFill(color));
        Port::GP0.write(Port::Command::GP0::TopLeftPosition(pos.x, pos.y));
        Port::GP0.write(Port::Command::GP0::WidthHeight(size.width, size.height));
    }

    static void reset_cmd_buffer() {
        Port::GP1.write(Port::Command::GP1::ResetCMDBufer());
    }

    static void wait_ready_for_CMD() {
        while(!Port::GPUSTAT.ref().is(Port::GPUStatusRegister::GP0ReadyForCMD));
    }

    namespace DMA {
        static void wait() {
            while(::DMA::Port::GPU.channel_ctrl.ref().is(::DMA::Port::CHCHR::Busy));
        }

        static void end() {
            reset_cmd_buffer();
        }

        namespace Receive {
            static void prepare()
            {
                Port::GP1.write(Port::Command::GP1::DMADirection(Port::DMADirection::CPU2GPU));
                reset_cmd_buffer();
            }

            static void set_src(uintptr_t adr) {
                ::DMA::Port::GPU.adr.ref().set_value(static_cast<uint32_t>(adr), ::DMA::Port::MADR::MemoryAdr);
            }

            static void set_dst(const PositionU16& position, const SizeU16& size) {

                wait_ready_for_CMD();
                Port::GP0.write(Port::Command::GP0::CPU2VRAM_Blitting());
                Port::GP0.write(Port::Command::GP0::TopLeftPosition(position.x, position.y));
                Port::GP0.write(Port::Command::GP0::WidthHeight(size.width, size.height));
            }

            static void start(uint16_t blockCount, uint16_t wordsPerBlock = 0x10) {
                typedef ::DMA::Port::BCR::SyncMode1 SyncMode1;

                ::DMA::Port::GPU.block_ctrl.write(SyncMode1::with(SyncMode1::BlockSize.with(wordsPerBlock), SyncMode1::BlockAmount.with(blockCount)));
                ::DMA::Port::GPU.channel_ctrl.write(::DMA::Port::CHCHR::StartGPUReceive());
            }
        }
    }
}

#endif //!__JABYENGINE_INTERNAL_GPU_HPP__